/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file clock_device.h
 */

#ifndef __CLOCK_DEVICE_H__
#define __CLOCK_DEVICE_H__

#include <stdint.h>
#include "hdf.h"

/**
 * @brief Initialize crystal clock.
 *
 * The clock source will be switched to crystal clock by this function.
 */
int32_t Clock_devCrystalClockInit(void);

/**
 * @brief Initialize RC clock.
 *
 * The clock source will be switched to RC clock by this function.
 */
void Clock_devRCClockInit(void);

/**
 * @brief Initialize 10K RC clock.
 *
 * The clock source will be switched to 10K RC by this function.
 */
void Clock_devRC10KClockInit(void);

/**
 * @brief Calibrate RC clock.
 */
void Clock_devRCCalibration(void);

/**
 * @brief Enable crystal clock.
 *
 * This function doesn't switch clock source.
 */
void Clock_devEnableCrystalClock(void);

/**
 * @brief Disable crystal clock.
 *
 * This function doesn't switch clock source.
 */
void Clock_devDisableCrystalClock(void);

/**
 * @brief Register a callback function to clock interrupt.
 *
 * @param cb The pointer to callback function.
 */
int32_t Clock_devRegisterIRQ(void *cb);

/**
 * @brief Un-register a callback function to clock interrupt.
 *
 * @param cb The pointer to callback function.
 */
int32_t Clock_devUnregisterIRQ(void *cb);

#endif /* __CLOCK_DEVICE_H__ */