/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file flash_device.h
 */

#ifndef __FLASH_DEVICE_H__
#define __FLASH_DEVICE_H__

#include <stdint.h>
#include "hdf.h"

/**
 * @brief Erase a block in flash memory and wait for completion
 *
 * @param addr The starting address of the block
 * @return none
 */
__ramfunc void Flash_devEraseBlock(uint32_t addr);

/**
 * @brief Erase a sector flash memory and wait for completion
 *
 * @param addr: starting address of the block
 * @return none
 */
__ramfunc void Flash_devEraseSector(uint32_t addr);

/**
 * @brief Copy a data block from RAM to flash memory.
 *
 * @param src: source address in RAM
 * @param des: destination address in flash memory
 * @param size: size of data in bytes
 * @return none
 */
__ramfunc void Flash_devCopyToFlash(uint8_t *src, uint8_t *des, uint32_t size);

#endif /* __FLASH_DEVICE_H__ */