/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file led_device.h
 */

#ifndef __LED_DEVICE_H__
#define __LED_DEVICE_H__

#include <stdint.h>
#include "hdf.h"

/**
 * @brief A structure to represent LED device.
 */
typedef const struct LED_Device_t {
    void (*On)(uint8_t pos);
    void (*Off)(uint8_t pos);
    void (*SetLightLevel)(uint8_t pos, uint8_t level);
    const char *Name;
} LED_Device_t;

/**
 * @brief Init LED mode for a pin
 *
 * @param dev The pointer to LED_Device
 * @param pos The pin position on LED port
 * @return 0 for success or error code upon a failure
 */
int32_t LED_devInit(LED_Device_t *dev, uint8_t pos);

/**
 * @brief DeInit LED mode for a pin
 *
 * @param dev The pointer to LED_Device
 * @param pos The pin position on LED port
 * @return 0 for success or error code upon a failure
 */
int32_t LED_devDeInit(LED_Device_t *dev, uint8_t pos);

/**
 * @brief Turn On a LED
 *
 * @param dev The pointer to LED_Device
 * @param pos The pin position on LED port
 * @return 0 for success or error code upon a failure
 */
int32_t LED_devOn(LED_Device_t *dev, uint8_t pos);

/**
 * @brief Turn off LED
 *
 * @param dev The pointer to LED_Device
 * @param pos The pin position on LED port
 * @return 0 fo success or error code upon a failure
 */
int32_t LED_devOff(LED_Device_t *dev, uint8_t pos);

/**
 * @brief Set LED light level
 *
 * @param dev The pointer to LED_Device
 * @param pos The pin position on LED port
 * @param level The light level
 * @return 0 fo success or error code upon a failure
 */
int32_t LED_devSetLightLevel(LED_Device_t *dev, uint8_t pos, uint8_t level);

/**
 * @brief Request a LED Device
 *
 * @param name The name string of LED device.
 * @return The pointer to the LED device instance.
 */
LED_Device_t *LED_devRequest(const char *name);

#endif /* __LED_DEVICE_H__ */