/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file pir_device.h
 */

#ifndef __PIR_DEVICE_H__
#define __PIR_DEVICE_H__

/**
 * @brief A structure to represent PIR device
 */
typedef struct {
    IRQn_Type IRQ;
    int32_t (*Init)(uint8_t clkdiv, uint8_t etime, uint8_t itime, uint8_t atten);
    int32_t (*EnableWakeup)(void *cb);
    int32_t (*DisableWakeup)(void);
    int32_t (*Disable)(void);
} PIR_Device_t;

typedef void (*pir_cb_func_t)(PIR_Device_t *dev);

/**
 * @brief Request A PIR Device
 *
 * @return The pointer to the RTC device instance.
 */
PIR_Device_t *PIR_devRequest(void);

/**
 * @brief Initialize PIR settings.
 *
 * @param dev The pointer to PIR device.
 * @param clkdiv The setting for clock divider. The valid values: 8,16,24,32
 * @param etime The event time in decisecond.
 * @param itime The inhibit time in seconds.
 * @param atten The attenuation in dB. The valid value 0 to 26 dB.
 * @return 0 or error code upon a failure.
 */
int32_t PIR_devInit(PIR_Device_t *dev, uint8_t clkdiv, uint8_t etime, uint8_t itime, uint8_t atten);

/**
 * @brief Enable PIR Wakeup.
 *
 * @param dev The pointer to PIR device.
 * @param cb The pointer to the call back function for the wakeup interrupt.
 * @return 0 or error code upon a failure.
 */
int32_t PIR_devEnableWakeup(PIR_Device_t *dev, void *cb);

/**
 * @brief Disable PIR Wakeup.
 *
 * @param dev The pointer to PIR device.
 * @return 0 or error code upon a failure.
 */
int32_t PIR_devDisableWakeup(PIR_Device_t *dev);

/**
 * @brief Disable PIR.
 *
 * @param dev The pointer to PIR device.
 * @return 0 or error code upon a failure.
 */
int32_t PIR_devDisable(PIR_Device_t *dev);

#endif /* __PIR_DEVICE_H__ */
