/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file rf_codec.h
 */

#ifndef __RF_CODEC_H__
#define __RF_CODEC_H__

#include "stdint.h"
     
/**
 * @brief The enum for the duty-cycle modulation format.
 */
typedef enum {
    DCM_FMT_NONE,
    DCM_FMT_3BIT,  /*!< 2/3 , 1/3 format, 0 bit for 110 and 1 bit for 100 */
    DCM_FMT_4BIT,  /*!< 3/4 , 1/4 format, 0 bit for 1110 and 1 bit for 1000 */
} DCM_FMT_t;

/**
 * @brief Encode data.
 *
 * @param mode The format for the data to be encoded.
 * @param src The pointer to the data to be encoded.
 * @param des The pointer to the decoded data to be saved.
 * @param len The length bytes of the source data in number of bytes.
 * @return 0 for success or error code upon a failure.
 * The function doesn't do memory boundary check, the caller needs to ensure the
 * both src and des pointing to sufficient memory space. 
 */
int32_t DCM_Encoder(DCM_FMT_t mode, uint8_t *src, uint8_t *des, uint8_t len);

#endif /* __RF_CODEC_H__ */