/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file gpio_device.c
 */

#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include "gpio_device.h"


#ifdef GPIOA_SFRS
/**
 * @brief The instances of GPIOA device
 */
static GPIO_Device_t GPIOA_Device = {
    .EnablePullUp = GPIOA_EnablePullUp,
    .EnablePullDown = GPIOA_EnablePullDown,
    .EnableOutput = GPIOA_EnableOutput,
    .EnableInput = GPIOA_EnableInput,
    .EnableInterrupt = GPIOA_EnableInterrupt,
    .Set = GPIOA_Set,
    .Read = GPIOA_Read,
    .EnablePortPullUp = GPIOA_EnablePortPullUp,
    .EnablePortPullDown = GPIOA_EnablePortPullDown,
    .EnablePortOutput = GPIOA_EnablePortOutput,
    .EnablePortInput = GPIOA_EnablePortInput,
    .EnablePortInterrupt = GPIOA_EnablePortInterrupt,
    .PortSet = GPIOA_PortSet,
    .PortRead = GPIOA_PortRead,
    .IRQ = PIN_IRQn,
    .Name = "GPIOA",
};
#endif
#ifdef GPIOB_SFRS
/**
 * @brief The instances of GPIOB device
 */
static GPIO_Device_t GPIOB_Device = {
    .EnablePullUp = GPIOB_EnablePullUp,
    .EnablePullDown = GPIOB_EnablePullDown,
    .EnableOutput = GPIOB_EnableOutput,
    .EnableInput = GPIOB_EnableInput,
    .EnableInterrupt = GPIOB_EnableInterrupt,
    .Set = GPIOB_Set,
    .Read = GPIOB_Read,
    .EnablePortPullUp = GPIOB_EnablePortPullUp,
    .EnablePortPullDown = GPIOB_EnablePortPullDown,
    .EnablePortOutput = GPIOB_EnablePortOutput,
    .EnablePortInput = GPIOB_EnablePortInput,
    .EnablePortInterrupt = GPIOB_EnablePortInterrupt,
    .PortSet = GPIOB_PortSet,
    .PortRead = GPIOB_PortRead,
    .IRQ = PIN_IRQn,
    .Name = "GPIOB",
};
#endif
#ifdef GPIOC_SFRS
/**
 * @brief The instances of GPIOC device
 */
static GPIO_Device_t GPIOC_Device = {
    .EnablePullUp = GPIOC_EnablePullUp,
    .EnablePullDown = GPIOC_EnablePullDown,
    .EnableOutput = GPIOC_EnableOutput,
    .EnableInput = GPIOC_EnableInput,
    .EnableInterrupt = NULL,
    .Set = GPIOC_Set,
    .Read = GPIOC_Read,
    .EnablePortPullUp = GPIOC_EnablePortPullUp,
    .EnablePortPullDown = GPIOC_EnablePortPullDown,
    .EnablePortOutput = GPIOC_EnablePortOutput,
    .EnablePortInput = GPIOC_EnablePortInput,
    .EnablePortInterrupt = NULL,
    .PortSet = GPIOC_PortSet,
    .PortRead = GPIOC_PortRead,
    .IRQ = PIN_IRQn,
    .Name = "GPIOC",
};
#endif
#ifdef GPIOD_SFRS
/**
 * @brief The instances of GPIOB device
 */
static GPIO_Device_t GPIOD_Device = {
    .EnablePullUp = GPIOD_EnablePullUp,
    .EnablePullDown = GPIOD_EnablePullDown,
    .EnableOutput = GPIOD_EnableOutput,
    .EnableInput = GPIOD_EnableInput,
    .EnableInterrupt = GPIOD_EnableInterrupt,
    .Set = GPIOD_Set,
    .Read = GPIOD_Read,
    .EnablePortPullUp = GPIOD_EnablePortPullUp,
    .EnablePortPullDown = GPIOD_EnablePortPullDown,
    .EnablePortOutput = GPIOD_EnablePortOutput,
    .EnablePortInput = GPIOD_EnablePortInput,
    .EnablePortInterrupt = GPIOD_EnablePortInterrupt,
    .PortSet = GPIOD_PortSet,
    .PortRead = GPIOD_PortRead,
    .IRQ = PIN_IRQn,
    .Name = "GPIOD",
};
#endif
#ifdef GPIOE_SFRS
/**
 * @brief The instances of GPIOE device
 */
static GPIO_Device_t GPIOE_Device = {
    .EnablePullUp = GPIOE_EnablePullUp,
    .EnablePullDown = GPIOE_EnablePullDown,
    .EnableOutput = GPIOE_EnableOutput,
    .EnableInput = GPIOE_EnableInput,
    .EnableInterrupt = NULL,
    .Set = GPIOE_Set,
    .Read = GPIOE_Read,
    .EnablePortPullUp = GPIOE_EnablePortPullUp,
    .EnablePortPullDown = GPIOE_EnablePortPullDown,
    .EnablePortOutput = GPIOE_EnablePortOutput,
    .EnablePortInput = GPIOE_EnablePortInput,
    .EnablePortInterrupt = NULL,
    .PortSet = GPIOE_PortSet,
    .PortRead = GPIOE_PortRead,
    .IRQ = PIN_IRQn,
    .Name = "GPIOE",
};
#endif
#ifdef GPIOF_SFRS
/**
 * @brief The instances of GPIOE device
 */
static GPIO_Device_t GPIOF_Device = {
    .EnablePullUp = GPIOF_EnablePullUp,
    .EnablePullDown = GPIOF_EnablePullDown,
    .EnableOutput = GPIOF_EnableOutput,
    .EnableInput = GPIOF_EnableInput,
    .EnableInterrupt = NULL,
    .Set = GPIOF_Set,
    .Read = GPIOF_Read,
    .EnablePortPullUp = GPIOF_EnablePortPullUp,
    .EnablePortPullDown = GPIOF_EnablePortPullDown,
    .EnablePortOutput = GPIOF_EnablePortOutput,
    .EnablePortInput = GPIOF_EnablePortInput,
    .EnablePortInterrupt = NULL,
    .PortSet = GPIOF_PortSet,
    .PortRead = GPIOF_PortRead,
    .IRQ = PIN_IRQn,
    .Name = "GPIOF",
};
#endif

/**
 * @brief The table of GPIO device instances supported by the platform.
 */
const static GPIO_Device_t *GPIO_DeviceTable[] = {
#ifdef GPIOA_SFRS
    &GPIOA_Device,
#endif
#ifdef GPIOB_SFRS
    &GPIOB_Device,
#endif
#ifdef GPIOC_SFRS
    &GPIOC_Device,
#endif
#ifdef GPIOD_SFRS
    &GPIOD_Device,
#endif
#ifdef GPIOE_SFRS
    &GPIOE_Device,
#endif
#ifdef GPIOF_SFRS
    &GPIOF_Device,
#endif
    NULL,
};

typedef void (*func_ptr_t)(void);
static func_ptr_t cb_func = NULL;

int32_t GPIO_devSet(GPIO_Device_t *dev, uint8_t pos, uint8_t state)
{
    if (!dev && !dev->Set)
        return -EFAULT;
    dev->Set(pos, state);
    return 0;
}

int32_t GPIO_devRead(GPIO_Device_t *dev, uint8_t pos)
{
    if (!dev && !dev->Read)
        return -EFAULT;
    return dev->Read(pos);
}

int32_t GPIO_devInit(GPIO_Device_t *dev, uint8_t pos, uint32_t mode)
{
    if (!dev)
        return -EFAULT;

    if (dev->EnableInterrupt) {
        if (mode & GPIO_MODE_INTERRUPT)
            dev->EnableInterrupt(pos, true);
        else
            dev->EnableInterrupt(pos, false);
    } else if (mode & GPIO_MODE_INTERRUPT)
        return -ENODEV;

    if (dev->EnableInput) {
        if (mode & GPIO_MODE_INPUT)
            dev->EnableInput(pos, true);
        else
            dev->EnableInput(pos, false);
    } else if (mode & GPIO_MODE_INPUT)
        return -ENODEV;

    if (dev->EnableOutput) {
        if (mode & GPIO_MODE_OUTPUT)
            dev->EnableOutput(pos, true);
        else
            dev->EnableOutput(pos, false);
    } else if (mode & GPIO_MODE_OUTPUT)
        return -ENODEV;

    if (dev->EnablePullUp) {
        if (mode & GPIO_MODE_PULLUP)
            dev->EnablePullUp(pos, true);
        else
            dev->EnablePullUp(pos, false);
    } else if (mode & GPIO_MODE_PULLUP)
        return -ENODEV;

    if (dev->EnablePullDown) {
        if (mode & GPIO_MODE_PULLDOWN) 
            dev->EnablePullDown(pos, true);
        else
            dev->EnablePullDown(pos, false);
    } else if (mode & GPIO_MODE_PULLDOWN) 
        return -ENODEV;

    return 0;
}

int32_t GPIO_devRegisterIRQ(GPIO_Device_t *dev, void *cb)
{
    if (!dev)
        return -EFAULT;
    cb_func = (func_ptr_t)cb;
    NVIC_EnableIRQ(dev->IRQ);

    return 0;
}

int32_t GPIO_devUnregisterIRQ(GPIO_Device_t *dev, void *cb)
{
    if (!dev)
        return -EFAULT;
    cb_func = NULL;
    NVIC_DisableIRQ(dev->IRQ);

    return 0;
}

void PIN_Handler ( void )
{
    if (cb_func != NULL)
        cb_func();
}

GPIO_Device_t *GPIO_devRequest(const char *name)
{
    int8_t i;

    for (i = 0; i < sizeof(GPIO_DeviceTable)/sizeof(GPIO_DeviceTable[0]); i++) {
        if ((GPIO_DeviceTable[i] != NULL) &&
            (strncmp(name, GPIO_DeviceTable[i]->Name,
                    strlen(GPIO_DeviceTable[i]->Name)) == 0))
            return GPIO_DeviceTable[i];
    }
    return NULL;
}

int32_t GPIO_devInitPort(GPIO_Device_t *dev, uint8_t mask, uint32_t mode)
{
    if (!dev)
        return -EFAULT;

    if (dev->EnablePortInterrupt) {
        if (mode & GPIO_MODE_INTERRUPT)
            dev->EnablePortInterrupt(true, mask);
        else
            dev->EnablePortInterrupt(false, mask);
    } else if (mode & GPIO_MODE_INTERRUPT)
        return -ENODEV;

    if (dev->EnablePortInput) {
        if (mode & GPIO_MODE_INPUT)
            dev->EnablePortInput(true, mask);
        else
            dev->EnablePortInput(false, mask);
    } else if (mode & GPIO_MODE_INPUT)
        return -ENODEV;

    if (dev->EnablePortOutput) {
        if (mode & GPIO_MODE_OUTPUT)
            dev->EnablePortOutput(true, mask);
        else
            dev->EnablePortOutput(false, mask);
    } else if (mode & GPIO_MODE_OUTPUT)
        return -ENODEV;

    if (dev->EnablePortPullUp) {
        if (mode & GPIO_MODE_PULLUP)
            dev->EnablePortPullUp(true, mask);
        else
            dev->EnablePortPullUp(false, mask);
    } else if (mode & GPIO_MODE_PULLUP)
        return -ENODEV;

    if (dev->EnablePortPullDown) {
        if (mode & GPIO_MODE_PULLDOWN) 
            dev->EnablePortPullDown(true, mask);
        else
            dev->EnablePortPullDown(false, mask);
    } else if (mode & GPIO_MODE_PULLDOWN) 
        return -ENODEV;

    return 0;
}

int32_t GPIO_devPortSet(GPIO_Device_t *dev, uint8_t mask, uint8_t val)
{
    if (!dev && !dev->PortSet)
        return -EFAULT;
    dev->PortSet(val, mask);
    return 0;
}

int32_t GPIO_devPortRead(GPIO_Device_t *dev, uint8_t mask)
{
    if (!dev && !dev->Read)
        return -EFAULT;
    return dev->PortRead(mask);
}