/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file pmu_device.c
 */

#include "pmu_device.h"

typedef void (*pmu_cb_func_t)(void);
pmu_cb_func_t brownout_cb = NULL;


static void set_brownout_threshold(uint32_t volt)
{
    if (volt < 2000)
        f_PMU_SetBrownOutLevel(E_PMU_PMUBOR_BOUTVALUE_2V0);
    else if (volt < 2200)
        f_PMU_SetBrownOutLevel(E_PMU_PMUBOR_BOUTVALUE_2V2);
    else if (volt < 2400)
        f_PMU_SetBrownOutLevel(E_PMU_PMUBOR_BOUTVALUE_2V4);
    else
        f_PMU_SetBrownOutLevel(E_PMU_PMUBOR_BOUTVALUE_2V6);   
}

void PMU_devSleepLowPwr(PMU_WAKEUP_TIME_t time)
{
    if (time > PMU_WAKEUP_TIME_MAX)
        return;
    f_PMU_WakeupTime(PMU_WAKEUP_TIME_tbl[time].manissa,
                     PMU_WAKEUP_TIME_tbl[time].exp);
    f_PMU_SleepContinue();
}

void PMU_devSleepLowPwrReboot(PMU_WAKEUP_TIME_t time)
{
    if (time > PMU_WAKEUP_TIME_MAX)
        return;
    f_PMU_WakeupTime(PMU_WAKEUP_TIME_tbl[time].manissa,
                     PMU_WAKEUP_TIME_tbl[time].exp);
    f_PMU_SleepReboot();
}

void PMU_devBrownOutInit(PMU_BROWNOUT_MODE_t mode, uint32_t volt, void *cb)
{
    switch (mode) {
    case PMU_BROWNOUT_RESET:
        brownout_cb = NULL;
        f_PMU_EnableBrownOutReset();
        f_PMU_DisableBrownOutInterrupt();
        set_brownout_threshold(volt);
        NVIC_DisableIRQ(BrownOut_IRQn);
        f_PMU_EnableBrownOut();
        break;

    case PMU_BROWNOUT_INTERRUPT:
        brownout_cb = (pmu_cb_func_t)cb;
        f_PMU_EnableBrownOutInterrupt();
        f_PMU_DisableBrownOutReset();
        set_brownout_threshold(volt);
        NVIC_EnableIRQ(BrownOut_IRQn);
        f_PMU_EnableBrownOut();
        break;

    case PMU_BROWNOUT_NONE:
        brownout_cb = NULL;
        f_PMU_DisableBrownOut();
        NVIC_DisableIRQ(BrownOut_IRQn);
    default:
        break;
    };
}

void PMU_devSetPowerModeBattery(void)
{
    f_PMU_SetPowerMode(E_PMU_POWER_MODE_BATTERY);
}

void PMU_devSetPowerModeLDO(void)
{
    f_PMU_SetPowerMode(E_PMU_POWER_MODE_LDO);
}

void BrownOut_Handler(void)
{
    if (brownout_cb != NULL)
        brownout_cb();
}
