/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file wdt_device.c
 */

#include "wdt_device.h"
#include "errno.h"

/**
 * @brief The instance of WDT device
 */
#ifdef WDT_SFRS
WDT_State_t WDT_State =
{
    .cb_func = NULL, .tick = 0,
};

const static WDT_Device_t WDT_Device =
{
    .Enable = f_WDT_Enable,
    .Disable = f_WDT_Disable,
    .ResetEnable = f_WDT_ResetEnable,
    .ResetDisable = f_WDT_ResetDisable,
    .SetPrescaler = f_WDT_SetPrescaler,
    .IsReset = f_WDT_IsReset,
    .Clear = f_WDT_Clear,
    .IRQ = WATCHDOG_IRQn,
    .State = &WDT_State,
};    
#endif

WDT_Device_t *WDT_devRequest(void)
{
#ifdef WDT_SFRS
    return &WDT_Device;
#else
    return NULL:
#endif
}

int32_t WDT_devIsReset(WDT_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return dev->IsReset();
}

int32_t WDT_devClear(WDT_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    dev->Clear();
    return 0;   
}

void Watchdog_Handler( void )
{
    if (WDT_State.cb_func != NULL)
        WDT_State.cb_func((WDT_Device_t *)&WDT_Device);
}

int32_t WDT_devInit(WDT_Device_t *dev, uint8_t mode, uint8_t time, void *cb)
{
    if (!dev)
        return -EFAULT;

    switch(mode) {
    case WDT_MODE_INTERRUPT:
        dev->ResetDisable();
        dev->State->cb_func = (wdt_cb_func_t)cb;
        dev->SetPrescaler(time);
        dev->Clear();
        NVIC_EnableIRQ(dev->IRQ);
        dev->Enable();
        break;

    case WDT_MODE_RESET:
        dev->ResetEnable();
        dev->State->cb_func = NULL;
        dev->SetPrescaler(time);
        dev->Clear();
        NVIC_DisableIRQ(dev->IRQ);
        dev->Enable();
        break;

    case WDT_MODE_DISABLE:
    default:
        dev->State->cb_func = NULL;
        NVIC_DisableIRQ(dev->IRQ);
        dev->Disable();
        break;
    }
    return 0;
}

