/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file led_sfrs.h
 */

#ifndef __LED_SFRS_H__
#define __LED_SFRS_H__

#define E_LED_BITWIDTH        2

/**
 * @brief A structure to represent LED configuration.
 */
typedef struct {
    uint8_t ISEL:4;
    uint8_t IR:1;
    uint8_t LEDEN:1;
    uint8_t :2;
} LED_CONFIG_t;

/**
 * @brief A structure to represent Special Function Registers for LED.
 */
typedef struct {
    uint8_t CTRL;
    uint8_t RESERVED0[98215];
    LED_CONFIG_t LEDCFG[E_LED_BITWIDTH];
} LED_SFRS_t;

/**
 * @brief The starting address of LED SFRS.
 */
#define LED_SFRS ((__IO LED_SFRS_t *) (0x50000062))

/**
 * @brief Turn on LED
 *
 * @param pos The pin position on the LED port.
 */
void f_LED_On(uint8_t pos);

/**
 * @brief Turn off LED
 *
 * @brief @param pos The pin position on the LED port.
 */
void f_LED_Off(uint8_t pos);

/**
 * @brief Set LED light level
 *
 * @param pos The pin position on the LED port.
 * @param level The light level.
 */
void f_LED_SetLightLevel(uint8_t pos, uint8_t level);

#endif /* __LED_SFRS_H__ */