/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file pir_sfrs.h
 */

#ifndef __PIR_SFRS_H__
#define __PIR_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for PIR.
 */
typedef struct {
    union {
        struct {
            uint16_t PIRC:10;
            uint16_t PIRDIV:2;
            uint16_t INHB:4;
        };
        uint16_t HWORD;
    } CTRL;
    union {
        struct {
            uint8_t ENABLE:1;
            uint8_t :7;
        };
        uint8_t BYTE;
    } ENABLE;
    uint8_t RESERVED[98234];
    union {
        struct {
            uint8_t :4;
            uint8_t PROGATT:4;
        };
        uint8_t BYTE;
    } ATTEN;
} PIR_SFRS_t;

/**
 * @brief The starting address of SFRS for PIR.
 */
#define PIR_SFRS ((__IO PIR_SFRS_t *) (0x50000050))

#define E_PIR_DIVIDER_STEPSIZE          8
#define E_PIR_ATTEN_STEPSIZE            2
#define PIR_DIVIDER_NOT_IN_RANGE(x) (x > 4)
#define PIR_COUNTER_NOT_IN_RANGE(x) (x > 1023)
#define PIR_INHB_NOT_IN_RANGE(x)    (x < 1 || x > 15)
#define PIR_ATTEN_NOT_IN_RANGE(x)   (x > 15)

void PIR_Enable(void);

void PIR_Disable(void);

void PIR_SetCounter(uint16_t val);

void PIR_SetDivider(uint8_t val); 

void PIR_SetInhibit(uint8_t val);

void PIR_SetAtten(uint8_t val);

#endif /* __PIR_SFRS_H__ */
