/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file rx_sfrs.h
 */

#ifndef __RX_SFRS_H__
#define __RX_SFRS_H__

#define RX_BUFF_SIZE    16

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// RF Receiver Definition
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
#define	RX_BASE		(ASIC_7B_BASE + 0x20)
#define	RX_SAP_BASE	(ASIC_16B_BASE + 0x1000)
#define	TRIM_SAP_BASE	(ASIC_16B_BASE + 0x8004)


#define	RFEN	0x80	//RF is ON 
#define RFOFF   0x00    //RF is OFF 

#define	SNIFEN	0x40	//Sniffer enable
#define SNIFDIS 0x00    //Sniffer disable 
#define	MSGRDY	0x20	//Define message ready: Reading returns one if 
			//a complete message is available and the receiver 
			//is in armed mode, otherwise returns zero.  
			//Write is ignored unless in armed mode, where 
			//writing a zero causes the receiver to leave 
			//the armed state and start decoding messages again.

#define	AGCOVF	0x10	//AGC overflow flag: Returns one if an overflow 
			//(signal too large) has occurred in the receiver.  
			//Intended for use in continuous mode, where the 
			//micro may want to reduce the gain setting. 

#define	SLICEROUT	0x02	//Slicer output
#define	RFSLEEP		0x01	//RF reception in sleep

#define	LOWBPS		0x80	// Count bit times at half rate
#define	NORMALBPS	0x00	// NOrmal BPS

#define	AGCEN	0x80	// AGC enabled or fixed gain (disabled)
#define	AGCDIS	0x00

// AGC_TRM [6...0]
//On read, returns the AGC bits being sent to the analog receiver.  On write, it 
//sets the values of a register which controls the behavior of the AGC system.  
//As such, typically read will return different value than had been written.
//The control register behavior depends on AGC_EN.  If AGC_EN is zero, then the 
//bits in the control register are used to directly control the AGC trim of the 
//analog blocks.  
//If AGC_EN is one, then the register controls several aspects of the AGC controller:
//AGC_EN[4] controls the step size of the AGC gain reduction.  
//If this bit is zero then the step size is 3dB per overflow, if one then the step 
//size is 6dB.
//AGC_EN[3:0] controls the default maximum gain, where the gain is notionally 
//3dB increased for each increase of this number.  Numbers up to 1010 are valid.  
//Numbers higher than 1010 will give unpredictable results.



#define ALPHA00   0x00        //Delay timer of the slicer 
#define ALPHA01   (0x01<<6)
#define ALPHA02   (0x02<<6)
#define ALPHA03   (0x03<<6)
#define BETA00    0x00        //Attack time of the slicer
#define BETA01    (0x01<<4)
#define BETA02    (0x02<<4)
#define BETA03    (0x03<<4)
#define FT00      0x00        //Time to allow slicer to fast bias
#define FT01      (0x01<<2)
#define FT02      (0x02<<2)
#define FT03      (0x03<<2)
#define SDR00     0x00        //Symbol Decimation rate
#define SDR01     0x01
#define SDR02     0x02
#define SDR03     0x03


#define	PLL32	(0x00<<6)		//Controls the time to wait for PLL to bias, measured in clock cycles of the Fxo/64:
#define	PLL48	(0x01<<6)		//00 => 32 cycles (570us for 3.58MHz crystal)
#define	PLL64	(0x02<<6)		//01 => 48 cycles (860us for 3.58 MHz crystal)
#define	PLL128	(0x03<<6)		//10 => 64 cycles (1.15ms for 3.58 MHz crystal)
					//11 => 128 cycles (2.29ms for 3.58MHz crystal)
#define	RFSLEEP4	(0x00<<3)	//Controls the time sleep between sniff cycles, measured in clock cycles of the fXO/64:
#define	RFSLEEP6	(0x01<<3)	//000 => 4*1024 cycles (73ms for 3.58MHz crystal)
#define	RFSLEEP8	(0x02<<3)	//001 => 6*1024 cycles (110ms for 3.58 MHz crystal)
#define	RFSLEEP10	(0x03<<3)	//010 => 8*1024 cycles (146ms for 3.58MHz crystal)
#define	RFSLEEP12	(0x04<<3)	//011 => 10*1024 cycles (183ms for 3.58MHz crystal)
#define	RFSLEEP14	(0x05<<3)	//100 => 12*1024 cycles (220ms for 3.58MHz crystal)
#define	RFSLEEP32	(0x06<<3)	//101 => 14*1024 cycles (256ms for 3.58 MHz crystal)
#define	RFSLEEP128	(0x07<<3)	//110 => 32*1024 cycles (586ms for 3.58MHz crystal)
								//111 => 128*1024 cycles (2.34s for 3.58MHz crystal)
#define	WAKET8		0x00		//Controls the time to stay awake after seeing a valid guardband, measured in clock cycles of the fXO/64:
#define	WAKET10		0x01		//000 => 8*1024 cycles (146ms for 3.58MHz crystal)
#define	WAKET12		0x02		//001 => 10*1024 cycles (183ms for 3.58 MHz crystal)
#define	WAKET14		0x03		//010 => 12*1024 cycles (220ms for 3.58MHz crystal)
#define	WAKET16		0x04		//011 => 14*1024 cycles (256ms for 3.58MHz crystal)
#define	WAKET18		0x05		//100 => 16*1024 cycles (293ms for 3.58MHz crystal)
#define	WAKET24		0x06		//101 => 18*1024 cycles (330ms for 3.58 MHz crystal)
#define	WAKET32		0x07		//110 => 24*1024 cycles (440ms for 3.58MHz crystal)
								//111 => 32*1024 cycles (586ms for 3.58MHz crystal)

								
#define RTSUP      0x00        //Selects a source for real-time output:
#define RTDE      (0x01<<6)    //00 => supervisor clock
#define RTSLICER  (0x02<<6)    //01 => decimator output (serialized stream)
#define RTPLL     (0x03<<6)    //10 => slicer output
                               //11 => PLL_EN (high if analog blocks enabled)

#define RFONEN    (0x01<<5)
#define RFONDIS    0x00
//snifferNE [3...0]


#define	HISDEN	0x80		//Controls image reject mixer to control whether
							//to use high-side or low side mixing.

#define	FEN		0x40		//If one then the PLL is used in fractional-N mode,
							//otherwise it is integer-N

#define	CPT00	0x00		//Charge pump current trim (settling time)
#define	CPT01	(0x01<<4)	//
#define	CPT02	(0x02<<4)	//
#define	CPT03	(0x03<<4)	//


#define	DIV_LO2_15	(0x00<<6)		//Controls the divide value of second LO. Divide value =
#define	DIV_LO2_16	(0x01<<6)		//16 +DIV_LO2[1:0]
#define	DIV_LO2_17	(0x02<<6)	
#define	DIV_LO2_18	(0x03<<6)	
#define	BBGAIN0dB	(0x00<<4)		//Base band gain 00 = 0dB; 01 = 6dB; 10 = 12dB; 11 =
#define	BBGAIN6dB	(0x01<<4)		//18dB
#define	BBGAIN12dB	(0x02<<4)	
#define	BBGAIN18dB	(0x03<<4)	
#define	LNADR1K		(0x01<<3)		//LNA drain resistor 0 = 2Kohms, 1 = 1Kohm
#define	LNADR2K 	(0x00<<3)		//
#define	LNABIAS0uA	(0x00)			//LNA bias current. 000 = 0; 001 = 200uA; 010 = 500uA;
#define	LNABIAS200uA	(0x01)		//011 = 700uA; 100 = 1mA; 101 = 1.2mA; 110 = 1.5mA
#define	LNABIAS500uA	(0x02)		//111 = 1.7mA
#define	LNABIAS700uA	(0x03)		
#define	LNABIAS1mA	(0x04)		
#define	LNABIAS1p2mA	(0x05)		
#define	LNABIAS1p5mA	(0x06)
#define	LNABIAS1p7mA	(0x07)
								
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// RF Receiver Structure
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
typedef union
{
  __IO uint32_t WORD[RX_BUFF_SIZE/4];
  __IO uint16_t HWORD[RX_BUFF_SIZE/2];
  __IO uint8_t  BYTE[RX_BUFF_SIZE];
} RFRX_DATAType; //RF Data received

typedef struct
{
  __IO uint8_t NUMB;  //RF RX data (received) size in bits
  __IO uint8_t STATUS;  //RF RX status
}RFRXType;

typedef struct
{
  __IO uint8_t NBMIN;    //RF Bit control
  __IO uint8_t AGCCTRL;  //RF AGC Control
  __IO uint8_t SLCCTRL;  //RF SLICER Control
  __IO uint8_t SYSTIME;  //RF System Time Control
  __IO uint16_t DCDTIME;  //RF Decoder time control
  __IO uint8_t SNIFMODE; //RF sniffer mode control
}RFRX_CTRLType;

typedef struct
{
  __IO uint8_t AGCMON;    //RF AGC monitor
  __IO uint16_t SIGI;  //RF I channel DC cal. value
  __IO uint16_t SIGQ;  //RF Q channel DC cal. value
  __IO uint16_t SLCHI;  //RF peak detector high
  __IO uint16_t SLCLO;  //RF peak detector low
}RFRX_MONTRType;

typedef union
{
   __IO uint32_t WORD; 
   __IO uint16_t HWORD[2];                            
   struct 
   {
     __IO uint8_t NX;     // RF NX Bits
     __IO uint8_t NF;     // RF NF Bits
     __IO uint8_t FETRIM0;  // RF Control 0
     __IO uint8_t FETRIM1;  // RF Control 1			              
   }BYTE;                         					   
}  RFRX_PLLType;   
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// RF Receiver Related Registers
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	

#define RFRX_DATA  ((RFRX_DATAType *) (RX_BASE + 0x00))	//RF RX Data received
#define RFRX       ((RFRXType *) (RX_BASE + 0x10))	//RF RX status return

#define RFRX_CTRL  ((RFRX_CTRLType *) (RX_SAP_BASE + 0x0000))	//RF RX control 

#define RFRX_MONTR  ((RFRX_MONTRType *) (RX_SAP_BASE + 0x0007))		//RF monitor


#define RFRX_PLL ((RFRX_PLLType *) (TRIM_SAP_BASE + 0x00))	// PLL Control to match the RX Freq



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// RF Receiver Function
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	

/**
* @brief RF State (ON/OFF)
*
* @return RFEN or RFOFF
* 
*/
  static __INLINE uint8_t RX_ReadState(void )
{
  return ( RFRX->STATUS & RFEN );
}

/**
* @brief Set RF State (ON/OFF)
*
* @param state The RX state, RFEN or RFOFF
* @return NONE
* 
*/
  static __INLINE void RX_SetState( uint8_t state )
{
  if ( state == RFEN ) RFRX->STATUS |= RFEN;
  else  RFRX->STATUS &= ~RFEN ;
}


/**
* @brief RF Sniffer Control (ON/OFF)
*
* @param mode The sniffer mode: SNIFEN or SNIFDIS
* @return NONE
* 
*/
  static __INLINE void RX_SnifferMode( uint8_t mode )
{
  if ( mode == SNIFEN )	{ RFRX->STATUS |= SNIFEN; }
  else { RFRX->STATUS &= ~SNIFEN; }
}



/**
* @brief RF Message Ready 
*
* @return MSGRDY or 0
* 
*/
  static __INLINE uint8_t RX_MessageReady( void )
{
  return ( RFRX->STATUS & MSGRDY );
}

/**
* @brief RF Reinit Decoder
*/
  static __INLINE void RX_ReinitDecoder(void )
{
  RFRX->STATUS= (RFRX->STATUS & (~MSGRDY)|AGCOVF);
}

/**
* @brief RF ACG Overflow
* @return AGCOVF or 0
* 
*/
  static __INLINE uint8_t RX_AgcOverflow( void )
{
  return ( RFRX->STATUS & AGCOVF );
}


/**
* @brief RF Bit Timing rate control
*
* @param rate: half rate or normal: LOWBPS or NORMALBPS	
* 
*/
  static __INLINE void RX_BitRateControl( uint8_t rate )
{
  if ( rate == LOWBPS )	{ RFRX_CTRL->NBMIN |= rate; }
  else { RFRX_CTRL->NBMIN &= ~LOWBPS; } 
}

/**
* @brief RF Bit Timing rate read
*
* @return LOWBPS or NORMALBPS
* 
*/
  static __INLINE uint8_t RX_ReadBitRateControl( void )
{
  return ( RFRX_CTRL->NBMIN & LOWBPS );
}

/**
* @brief Set RF Minimum number of bits.
*
* @param numbits The number of bits.
* 
*/
  static __INLINE void RX_SetMinBitNumber( uint8_t numbits )
{
  RFRX_CTRL->NBMIN &= LOWBPS; 
  RFRX_CTRL->NBMIN |= numbits; 	
}

/**
* @brief Read RF Minimum number of bits.
*
* @return number of bits from 0 to 127
*/
  static __INLINE uint8_t RX_ReadMinBitNumber( void )
{
  return ( RFRX_CTRL->NBMIN & 0x7F );	
}

/**
* @brief AGC control and trim
*
* @param mode: AGCEN or AGCDIS
* @param trim: [6...0] 
* 
*/
  static __INLINE void RX_AgcControl( uint8_t mode, uint8_t trim )
{
  RFRX_CTRL->AGCCTRL = (mode & AGCEN ) | trim;
}

/**
* @brief Slicer control
*
* @param alpha:
* @param beta:
* @param ft:
* @param sdr:
* 
*/
  static __INLINE void RX_SlicerControl( uint8_t alpha, uint8_t beta , uint8_t ft , uint8_t sdr )
{
  RFRX_CTRL->SLCCTRL = alpha | beta | ft | sdr;
}


/**
* @brief Set time to wait for PLL bias 
*
* @param btime: PLL32, PLL48, PLL64, PLL128
* 
*/
  static __INLINE void RX_SetPllBiasTime( uint8_t btime )
{
  RFRX_CTRL->SYSTIME = ( RFRX_CTRL->SYSTIME & 0x3F ) | btime;
}


/**
* @brief Read time to wait for PLL bias 
*
*/
  static __INLINE uint8_t RX_ReadPllBiasTime( void )
{
  return ( RFRX_CTRL->SYSTIME & 0xC0 );
}


/**
* @brief Set sleep time between sniffs
*
* @param stime
*/
  static __INLINE void RX_SetSleepTime( uint8_t stime )
{
  RFRX_CTRL->SYSTIME = ( RFRX_CTRL->SYSTIME & 0xC7 ) | stime;
}

/**
* @brief Read sleep time between sniffs
*
* @return sniff sleep time. (RFSLEEPn)
*
*/
  static __INLINE uint8_t RX_ReadSleepTime( void )
{
  return ( RFRX_CTRL->SYSTIME & 0x38 );
}

/**
* @brief Set wake time in sniffs
*
* @param wtime
*/
  static __INLINE void RX_SetWakeTime( uint8_t wtime )
{
  RFRX_CTRL->SYSTIME = ( RFRX_CTRL->SYSTIME & 0xF8 ) | wtime;
}

/**
* @brief Read wake time in sniffs
*
* @return Wake Time 
*/
  static __INLINE uint8_t RX_ReadWakeTime( void )
{
  return ( RFRX_CTRL->SYSTIME & 0x07 );
}

//DCDTIME  
//Min_Te [15...12]
//Max_Te [11...6]
//Min_Gb [5...0]

/**
* @brief Set Min_Te timer
*
* @param minte - The value to be used to calculate the minimum time for a bit 
*				 to be accepted as valid: = (1+Min_Te)*(12*16)/3.58e6
*/
  static __INLINE void RX_SetMinTe( uint8_t minte )
{
  RFRX_CTRL->DCDTIME = ( RFRX_CTRL->DCDTIME & 0x0FFF ) | ( minte << 12);
}

/**
* @brief Read Min_Te timer
*
* @return min_te 
*/
  static __INLINE uint8_t RX_ReadMinTe( void )
{
  return ( ( uint8_t ) ( RFRX_CTRL->DCDTIME >> 12 ) );
}

/**
* @brief Set Max_Te timer
*
* @param maxte - The value to be used to calculate the maximum time for a bit 
*				 to be accepted as valid: = (1 + Min_Te + Max_Te)*(12*16)/3.58e6
*/
  static __INLINE void RX_SetMaxTe( uint8_t maxte )
{
  RFRX_CTRL->DCDTIME = ( RFRX_CTRL->DCDTIME & 0xF03F ) | ( maxte << 6 );
}

/**
* @brief Read Max_Te timer
*
* @return max_te 
*/
  static __INLINE uint8_t RX_ReadMaxTe( void )
{
  return ( (uint8_t) (( RFRX_CTRL->DCDTIME >> 6 ) & 0x003F ));
}

/**
* @brief Set Min_Gb timer
*
* @param mingb - The value to be used to calculate the minimum time for a packet
*				 to be accepted as valid: = (1 + Min_Te + Max_Te + Min_Gb)*(12*16)/3.58e6
*/
  static __INLINE void RX_SetMinGb( uint8_t mingb )
{
  RFRX_CTRL->DCDTIME = ( RFRX_CTRL->DCDTIME & 0xFFC0 ) | ( mingb );
}

/**
* @brief Read Min_Gb timer
*
* @return mingb 
*/
  static __INLINE uint8_t RX_ReadMinGb(void )
{
  return ( (uint8_t) ( RFRX_CTRL->DCDTIME & 0x003F) );
}

/**
* @brief Set Sniffer mode timing
*
* @param rt - source for real-time output:
* RTSUP      supervisor clock
* RTDE       decimator output (serialized stream)   
* RTSLICER   slicer output
* RTPLL      PLL_EN (high if analog blocks enabled)
* @param rfall - RF always on or not (For test) RFONEN or RFONDIS
* @param sniff - Number of edges to check in each sniff cycle before committing 
*                to a long wake cycle.  When in sniff mode, the first SNIFF_NE 
*                edges are tested for valid timing.  If any one of these first 
*                edges are badly timed, then the receiver will go to sleep.
*/
  static __INLINE void RX_SetSnifRt( uint8_t rt, uint8_t rfall, uint8_t sniff )
{
  RFRX_CTRL->SNIFMODE = rt & RTPLL | (rfall & RFONEN) | sniff ;
}

/**
* @brief Read AGC gain value
*
* @return agc
*/
 static __INLINE uint8_t RX_ReadAgc( void )
{
  return ( RFRX_MONTR->AGCMON );
}

/**
* @brief RF Control 0
*
* @param rejection: Single side band demodulation with Weaver image
* @param rejection architecture --> HISDEN = 1  = high side band; HISDDIS = 0 = Low side band I
* @param fractional: FEN = fractional active, FDIS = integer active
* @param cpt - charge pump trim: CPT00 to CPT3
* @param lfrt - 0...15
*/
  static __INLINE void RX_Setup0( uint8_t rejection, uint8_t fractional, uint8_t cpt, uint8_t lfrt )
{
  RFRX_PLL->BYTE.FETRIM0 = (rejection & HISDEN) | (fractional &FEN) | cpt | lfrt;
}

/**
* @brief RF Control 1
*
* @param lodiv: DIV_LO2_16 to DIV_LO2_19
* @param bbg: BBGAIN0dB to BBGAIN18dB
* @param lnadr: LNADREN or LNADRDIS
* @param lnab - LNABIAS0uA to LNABIAS1p7mA
*/
  static __INLINE void RX_Setup1( uint8_t lodiv, uint8_t bbg, uint8_t lnadr, uint8_t lnab )
{
  RFRX_PLL->BYTE.FETRIM1 = lodiv | bbg | lnadr | lnab;
}

#endif /* __RX_SFRS_H__ */