/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file gpio_sfrs.c
 */

#include "hdf.h"

void GPIOD_EnablePullUp(uint8_t pos, bool bit)
{
    GPIOD_SFRS->CFG[pos].PULLUP = ~bit;
}

void GPIOD_EnablePullDown(uint8_t pos, bool bit)
{
    GPIOD_SFRS->CFG[pos].PULLDOWN = bit;
}

void GPIOD_EnableOutput(uint8_t pos, bool bit)
{
    GPIOD_SFRS->CFG[pos].OUTPUTEN = bit;
}

void GPIOD_EnableInput(uint8_t pos, bool bit)
{
    GPIOD_SFRS->CFG[pos].READEN = bit;
}

void GPIOD_EnableInterrupt(uint8_t pos, bool bit)
{
    if (bit)
        GPIOD_SFRS->INTEN |=  (1 << pos);
    else
        GPIOD_SFRS->INTEN &=  ~(1 << pos);
}

uint8_t GPIOD_Read(uint8_t pos)
{
    return GPIOD_SFRS->PORT >> pos & 1;
}

void GPIOD_Set(uint8_t pos, uint8_t bit)
{
    uint8_t readen[E_GPIOD_BITWIDTH];
    uint8_t i;
    
    for (i = 0; i < E_GPIOD_BITWIDTH; i++)
        readen[i] = GPIOD_SFRS->CFG[i].READEN;
    for (i = 0; i < E_GPIOD_BITWIDTH; i++)
        GPIOD_SFRS->CFG[i].READEN = 1;
    if (bit)
        GPIOD_SFRS->PORT |=  (1 << pos);
    else
        GPIOD_SFRS->PORT &=  ~(1 << pos);
    for (i = 0; i < E_GPIOD_BITWIDTH; i++)
        GPIOD_SFRS->CFG[i].READEN = readen[i];
}

void GPIOD_EnablePortPullUp(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOD_BITWIDTH; i++) {
        if (bit)
            GPIOD_SFRS->CFG[i].PULLUP &= ~mask;
        else
            GPIOD_SFRS->CFG[i].PULLUP |= mask;
    }
}

void GPIOD_EnablePortPullDown(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOD_BITWIDTH; i++) {
        if (bit)
            GPIOD_SFRS->CFG[i].PULLDOWN |= mask;
        else
            GPIOD_SFRS->CFG[i].PULLDOWN &= ~mask;
    }
}

void GPIOD_EnablePortOutput(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOD_BITWIDTH; i++) {
        if (bit)
            GPIOD_SFRS->CFG[i].OUTPUTEN |=  mask;
        else
            GPIOD_SFRS->CFG[i].OUTPUTEN &= ~mask;
    }
}

void GPIOD_EnablePortInput(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOD_BITWIDTH; i++) {
        if (bit)
            GPIOD_SFRS->CFG[i].READEN |= mask;
        else
            GPIOD_SFRS->CFG[i].READEN &= ~mask;
    }
}

void GPIOD_EnablePortInterrupt(bool bit, uint8_t mask)
{
    if (bit)
        GPIOD_SFRS->INTEN |= mask;
    else
        GPIOD_SFRS->INTEN &= ~mask;
}

void GPIOD_PortSet(uint8_t val, uint8_t mask)
{
    uint8_t readen[E_GPIOD_BITWIDTH];
    uint8_t i;
    
    for (i = 0; i < E_GPIOD_BITWIDTH; i++)
        readen[i] = GPIOD_SFRS->CFG[i].READEN;
    for (i = 0; i < E_GPIOD_BITWIDTH; i++)
        GPIOD_SFRS->CFG[i].READEN = 1;
    GPIOD_SFRS->PORT &= ~mask;
    GPIOD_SFRS->PORT |= (val & mask);
    for (i = 0; i < E_GPIOD_BITWIDTH; i++)
        GPIOD_SFRS->CFG[i].READEN = readen[i];
}

uint8_t GPIOD_PortRead(uint8_t mask)
{
    return GPIOD_SFRS->PORT & mask;
}




void GPIOE_EnablePullUp(uint8_t pos, bool bit)
{
    GPIOE_SFRS->CFG[pos].PULLUP = ~bit;
}

void GPIOE_EnablePullDown(uint8_t pos, bool bit)
{
    GPIOE_SFRS->CFG[pos].PULLDOWN = bit;
}

void GPIOE_EnableOutput(uint8_t pos, bool bit)
{
    GPIOE_SFRS->CFG[pos].OUTPUTEN = bit;
}

void GPIOE_EnableInput(uint8_t pos, bool bit)
{
    GPIOE_SFRS->CFG[pos].READEN = bit;
}

void GPIOE_EnableInterrupt(uint8_t pos, bool bit)
{
    if (bit)
        GPIOE_SFRS->INTEN |=  (1 << pos);
    else
        GPIOE_SFRS->INTEN &=  ~(1 << pos);
}

uint8_t GPIOE_Read(uint8_t pos)
{
    return GPIOE_SFRS->PORT >> pos & 1;
}

void GPIOE_Set(uint8_t pos, uint8_t bit)
{
    uint8_t readen[E_GPIOE_BITWIDTH];
    uint8_t i;
    
    for (i = 0; i < E_GPIOE_BITWIDTH; i++)
        readen[i] = GPIOE_SFRS->CFG[i].READEN;
    for (i = 0; i < E_GPIOE_BITWIDTH; i++)
        GPIOE_SFRS->CFG[i].READEN = 1;
    if (bit)
        GPIOE_SFRS->PORT |=  (1 << pos);
    else
        GPIOE_SFRS->PORT &=  ~(1 << pos);
    for (i = 0; i < E_GPIOE_BITWIDTH; i++)
        GPIOE_SFRS->CFG[i].READEN = readen[i];
}

void GPIOE_EnablePortPullUp(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOE_BITWIDTH; i++) {
        if (bit)
            GPIOE_SFRS->CFG[i].PULLUP &= ~mask;
        else
            GPIOE_SFRS->CFG[i].PULLUP |= mask;
    }
}

void GPIOE_EnablePortPullDown(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOE_BITWIDTH; i++) {
        if (bit)
            GPIOE_SFRS->CFG[i].PULLDOWN |= mask;
        else
            GPIOE_SFRS->CFG[i].PULLDOWN &= ~mask;
    }
}

void GPIOE_EnablePortOutput(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOE_BITWIDTH; i++) {
        if (bit)
            GPIOE_SFRS->CFG[i].OUTPUTEN |=  mask;
        else
            GPIOE_SFRS->CFG[i].OUTPUTEN &= ~mask;
    }
}

void GPIOE_EnablePortInput(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOE_BITWIDTH; i++) {
        if (bit)
            GPIOE_SFRS->CFG[i].READEN |= mask;
        else
            GPIOE_SFRS->CFG[i].READEN &= ~mask;
    }
}

void GPIOE_EnablePortInterrupt(bool bit, uint8_t mask)
{
    if (bit)
        GPIOE_SFRS->INTEN |= mask;
    else
        GPIOE_SFRS->INTEN &= ~mask;
}

void GPIOE_PortSet(uint8_t val, uint8_t mask)
{
    uint8_t readen[E_GPIOE_BITWIDTH];
    uint8_t i;
    
    for (i = 0; i < E_GPIOE_BITWIDTH; i++)
        readen[i] = GPIOE_SFRS->CFG[i].READEN;
    for (i = 0; i < E_GPIOE_BITWIDTH; i++)
        GPIOE_SFRS->CFG[i].READEN = 1;
    GPIOE_SFRS->PORT &= ~mask;
    GPIOE_SFRS->PORT |= (val & mask);
    for (i = 0; i < E_GPIOE_BITWIDTH; i++)
        GPIOE_SFRS->CFG[i].READEN = readen[i];
}

uint8_t GPIOE_PortRead(uint8_t mask)
{
    return GPIOE_SFRS->PORT & mask;
}




void GPIOF_EnablePullUp(uint8_t pos, bool bit)
{
    GPIOF_SFRS->CFG[pos].PULLUP = ~bit;
}

void GPIOF_EnablePullDown(uint8_t pos, bool bit)
{
    GPIOF_SFRS->CFG[pos].PULLDOWN = bit;
}

void GPIOF_EnableOutput(uint8_t pos, bool bit)
{
    GPIOF_SFRS->CFG[pos].OUTPUTEN = bit;
}

void GPIOF_EnableInput(uint8_t pos, bool bit)
{
    GPIOF_SFRS->CFG[pos].READEN = bit;
}

void GPIOF_EnableInterrupt(uint8_t pos, bool bit)
{
    if (bit)
        GPIOF_SFRS->INTEN |=  (1 << pos);
    else
        GPIOF_SFRS->INTEN &=  ~(1 << pos);
}

uint8_t GPIOF_Read(uint8_t pos)
{
    return GPIOF_SFRS->PORT >> pos & 1;
}

void GPIOF_Set(uint8_t pos, uint8_t bit)
{
    uint8_t readen[E_GPIOF_BITWIDTH];
    uint8_t i;
    
    for (i = 0; i < E_GPIOF_BITWIDTH; i++)
        readen[i] = GPIOF_SFRS->CFG[i].READEN;
    for (i = 0; i < E_GPIOF_BITWIDTH; i++)
        GPIOF_SFRS->CFG[i].READEN = 1;
    if (bit)
        GPIOF_SFRS->PORT |=  (1 << pos);
    else
        GPIOF_SFRS->PORT &=  ~(1 << pos);
    for (i = 0; i < E_GPIOF_BITWIDTH; i++)
        GPIOF_SFRS->CFG[i].READEN = readen[i];
}

void GPIOF_EnablePortPullUp(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOF_BITWIDTH; i++) {
        if (bit)
            GPIOF_SFRS->CFG[i].PULLUP &= ~mask;
        else
            GPIOF_SFRS->CFG[i].PULLUP |= mask;
    }
}

void GPIOF_EnablePortPullDown(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOF_BITWIDTH; i++) {
        if (bit)
            GPIOF_SFRS->CFG[i].PULLDOWN |= mask;
        else
            GPIOF_SFRS->CFG[i].PULLDOWN &= ~mask;
    }
}

void GPIOF_EnablePortOutput(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOF_BITWIDTH; i++) {
        if (bit)
            GPIOF_SFRS->CFG[i].OUTPUTEN |=  mask;
        else
            GPIOF_SFRS->CFG[i].OUTPUTEN &= ~mask;
    }
}

void GPIOF_EnablePortInput(bool bit, uint8_t mask)
{
    uint8_t i;

    for (i = 0; i < E_GPIOF_BITWIDTH; i++) {
        if (bit)
            GPIOF_SFRS->CFG[i].READEN |= mask;
        else
            GPIOF_SFRS->CFG[i].READEN &= ~mask;
    }
}

void GPIOF_EnablePortInterrupt(bool bit, uint8_t mask)
{
    if (bit)
        GPIOF_SFRS->INTEN |= mask;
    else
        GPIOF_SFRS->INTEN &= ~mask;
}

void GPIOF_PortSet(uint8_t val, uint8_t mask)
{
    uint8_t readen[E_GPIOF_BITWIDTH];
    uint8_t i;
    
    for (i = 0; i < E_GPIOF_BITWIDTH; i++)
        readen[i] = GPIOF_SFRS->CFG[i].READEN;
    for (i = 0; i < E_GPIOF_BITWIDTH; i++)
        GPIOF_SFRS->CFG[i].READEN = 1;
    GPIOF_SFRS->PORT &= ~mask;
    GPIOF_SFRS->PORT |= (val & mask);
    for (i = 0; i < E_GPIOF_BITWIDTH; i++)
        GPIOF_SFRS->CFG[i].READEN = readen[i];
}

uint8_t GPIOF_PortRead(uint8_t mask)
{
    return GPIOF_SFRS->PORT & mask;
}

void GPIO_ConfigUART(uint8_t mode, uint8_t swap, uint8_t invert)
{
    GPIO_MODE_SFRS->MDUART = mode;
    GPIO_MODE_SFRS->UPSWAP = swap;
    GPIO_MODE_SFRS->UTXPOL = invert;
}

void GPIO_ConfigSPI(uint8_t mode)
{
    GPIO_MODE_SFRS->MDSPI = mode;
}
