/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_adc.c
 */


#include "adc_device.h"
#include "clock_device.h"
#include "flash_device.h"
#include "gpio_device.h"
#include "irq_device.h"
#include "led_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "timer_device.h"
#include "wdt_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER_MS                30000                  /* 1 mS */
#define ADC_REF_H_DFLT          0x0F
#define ADC_REF_L_DFLT          0x00
#define ADC_REF_G_DFLT          0x0F

/************** Local Variables ***********************************************/
static SysTick_Device_t *systick = NULL;

/************** Global Variables **********************************************/
/************** Functions *****************************************************/

/************** MAIN ****************************************************/
void main_adc (void)
{
    int32_t ret;

    uint8_t adc_result0;
        
    /* Initialize system clock */
    Clock_devCrystalClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* Get GPIOE */
    GPIO_Device_t *gpioe = GPIO_devRequest("GPIOE");
    if(!gpioe)
        return;
    /* Initialize GPIOD.0 as interrupt input */
    GPIO_devInit(gpioe, 7, GPIO_MODE_INPUT |GPIO_MODE_PULLUP);

    ADC_Device_t *adc = ADC_devRequest("ADC");
    if(!adc)
        return;
    ret = ADC_devInit(adc, E_ADC_CLK_DIV64, 8, ADC_MODE_SINGLE | ADC_MODE_NORMAL | ADC_MODE_REF_VDD);
    if (ret)
        return;
    ret = ADC_devConfigReference(adc, ADC_REF_H_DFLT, ADC_REF_L_DFLT, ADC_REF_G_DFLT);
    if (ret)
        return;

    while (1) {
        ADC_devTake(adc, E_ADC_CHANNEL_GPIOE7, &adc_result0); 
        SysTick_devDelayMs(systick, 100);
   } /* while (1) */
} /* main() */

/***************************** End of File ************************************/