/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main.c
 */

#include "adc_device.h"
#include "clock_device.h"
#include "flash_device.h"
#include "gpio_device.h"
#include "uart_device.h"
#include "spi_device.h"
#include "irq_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "timer_device.h"
#include "wdt_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER0_Interval         512000                  /* 40ms */
#define TIMER1_Interval         TIMER0_Interval * 25    /* 1s */
#define TIMER2_Interval         TIMER0_Interval * 50    /* 2s */


/************** Local Variables ***********************************************/
static SysTick_Device_t *systick = NULL;

/************** Global Variables **********************************************/
/************** Functions *****************************************************/

/************** MAIN ****************************************************/
void main_demo (void)
{
    /* Initialize system clock */
    Clock_devRCCalibration();
    Clock_devRCClockInit();
    Clock_devEnableCrystalClock();

    /* Initialize power mode */
    PMU_devSetPowerModeLDO();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);
 
    while (1) {      
    } /* while (1) */
} /* main() */

/***************************** End of File ************************************/