/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_flash.c
 */

#include "clock_device.h"
#include "flash_device.h"
#include "led_device.h"
#include "systick_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)
#define TIMER_MS                30000                  /* 1 mS */

/************** Local Variables ***********************************************/
static LED_Device_t *led = NULL;
static SysTick_Device_t *systick = NULL;


/************** Global Variables **********************************************/
/************** Functions *****************************************************/

/************** MAIN ****************************************************/
void main_flash(void)
{
    uint32_t i;
    uint8_t flash_data[256];
    
    /* Initialize system clock */
    Clock_devCrystalClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* LED driver */
    led = LED_devRequest("LED");
    if(!led)
        return;
    /* Set LED light level to 80% */
    LED_devSetLightLevel(led, 0, 80);
    /* Turn on LED */
    LED_devOn(led, 0);

    for (i = 0; i < 256; i++)
        flash_data[i] = i;

    Flash_devEraseBlock(0x00010000);
    Flash_devCopyToFlash((uint8_t *)0x00010000, (uint8_t *)flash_data, 256);

    while (1) {
   } /* while (1) */
} /* main() */

/***************************** End of File ************************************/