/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_pir.c
 */


#include "adc_device.h"
#include "clock_device.h"
#include "flash_device.h"
#include "gpio_device.h"
#include "irq_device.h"
#include "led_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "timer_device.h"
#include "wdt_device.h"
#include "pir_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER_MS                30000                  /* 1 mS */

#define ADC_REF_H_DFLT          0x0F
#define ADC_REF_L_DFLT          0x00
#define ADC_REF_G_DFLT          0x0F

/************** Local Variables ***********************************************/
static LED_Device_t *led = NULL;
static SysTick_Device_t *systick = NULL;
static GPIO_Device_t *gpiod;
static GPIO_Device_t *gpioe;
static PIR_Device_t *pir;

static uint32_t pir_fired = 0;

/************** Global Variables **********************************************/
/************** Functions *****************************************************/
static void pir_wakeup_func(PIR_Device_t *pir)
{
    LED_devOff(led, 0);
    pir_fired = 1;
}

static void gpio_pin_func(void)
{
    /* Tracking pin changes for motion detection in active mode */
}

/************** MAIN ****************************************************/
void main_pir(void)
{
    uint32_t i;
        
    /* Initialize system clock as slow RC clock */
    Clock_devRC10KClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* Get GPIOE */
    gpioe = GPIO_devRequest("GPIOE");
    if(!gpioe)
        return;
    GPIO_devInit(gpioe, 6, GPIO_MODE_INPUT |GPIO_MODE_PULLUP);

        /* Get GPIOD */
    gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return;
    /* Initialize GPIOD.7 as interrupt input for PIR */
    GPIO_devInit(gpiod, 7, GPIO_MODE_INTERRUPT | GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 3, GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 4, GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 5, GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 6, GPIO_MODE_INPUT);
    /* Register pin toggling interrupt */
    GPIO_devRegisterIRQ(gpiod, (void *)&gpio_pin_func);

    /* LED driver */
    led = LED_devRequest("LED");
    if(!led)
        return;
    /* Set LED light level to 80% */
    LED_devSetLightLevel(led, 0, 80);
    /* Turn on LED */
    LED_devOn(led, 0);

    pir = PIR_devRequest();
    if (!pir)
        return;
    PIR_devInit(pir, 8, 1, 3, 0);
    PIR_devEnableWakeup(pir, (void *)&pir_wakeup_func);

    while (1) {
        if (pir_fired) {
            pir_fired = 0;
            LED_devOn(led, 0);
        }
        for (i = 0; i < 500; i++);
   } /* while (1) */
} /* main() */

/***************************** End of File ************************************/