/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_spi.c
 */

#include "clock_device.h"
#include "flash_device.h"
#include "gpio_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "spi_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)
#define BUFLEN                  4
#define SPI_READ_ADDR           0xA0
#define SPI_WRITE_ADDR          0x50

/************** Local Variables ***********************************************/
static SysTick_Device_t *systick = NULL;
static uint8_t spi_tx_data[BUFLEN] = {0x55, 0xAA, 0x0A, 0x05};
static uint8_t spi_rx_data[BUFLEN];

/************** Global Variables **********************************************/
/************** Functions *****************************************************/

/************** MAIN ****************************************************/
void main_spi (void)
{
    int32_t ret;

    /* Initialize system clock */
    Clock_devCrystalClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* Initialize GPIO pins for SPI mode */
    GPIO_Device_t *gpioe = GPIO_devRequest("GPIOE");
    if(!gpioe)
        return;
    GPIO_ConfigSPI(E_GPIO_PIN_MODE_ON);
    GPIO_devInit(gpioe, 1, GPIO_MODE_INPUT);
    GPIO_devInit(gpioe, 2, GPIO_MODE_OUTPUT);
    GPIO_devInit(gpioe, 3, GPIO_MODE_OUTPUT);
    GPIO_devInit(gpioe, 0, GPIO_MODE_OUTPUT);

    /* Initialize SPI driver */
    SPI_Device_t *spi = SPI_devRequest("SPI");
    if(!spi)
        return;
    SPI_Device_init_t spi_init = {      
        .Mode = SPI_DEV_MODE_MASTER,
        .ClockDiv = SPI_DEV_CLOCK_DIV32,
        .Phase = SPI_DEV_CLOCK_PHASE0,
        .Polarity = SPI_DEV_CLOCK_POLARITY0,
        .IntDlyCnt = 0,
    };
    ret = SPI_devInit(spi, &spi_init);
    if (ret)
        return;

    while (1) {
        ret = SPI_devWriteData(spi, SPI_WRITE_ADDR, spi_tx_data, BUFLEN);
        ret = SPI_devReadData(spi, SPI_READ_ADDR, &spi_rx_data[0], BUFLEN);
        SysTick_devDelayMs(systick, 500);
    } /* while (1) */
} /* main() */

/***************************** End of File ************************************/