/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_led.c
 */

#include "clock_device.h"
#include "gpio_device.h"
#include "irq_device.h"
#include "led_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "wdt_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER_MS                30000                  /* 1 mS */

/************** Local Variables ***********************************************/
static LED_Device_t *led;
static SysTick_Device_t *systick;
static WDT_Device_t *wdt;

/************** Global Variables **********************************************/
/************** Functions *****************************************************/
void wdt_func(void)
{
    WDT_devClear(wdt);
}

/************** MAIN ****************************************************/
void main_wdt(void)
{
    /* Initialize system clock */
    Clock_devCrystalClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* Get GPIOE */
    GPIO_Device_t *gpioe = GPIO_devRequest("GPIOE");
    if(!gpioe)
        return;
    GPIO_devInit(gpioe, 6, GPIO_MODE_INPUT |GPIO_MODE_PULLUP);

    /* LED driver */
    led = LED_devRequest("LED");
    if(!led)
        return;
    /* Set LED light level to 80% */
    LED_devSetLightLevel(led, 0, 80);
    /* Turn on LED */
    LED_devOn(led, 0);

    /* Watchdog timer example */
    wdt = WDT_devRequest();
    if(!wdt)
        return;
    
    /* If you want to use WDT_MODE_RESET, must be very careful that you have a way to come back
        Here we use GPIOE.6 as the safety guard for the test */
    if (GPIO_devRead(gpioe, 6))
        WDT_devInit(wdt, WDT_MODE_RESET, WDT_PRES_2PWR22, (void *)&wdt_func);
    else
        WDT_devInit(wdt, WDT_MODE_INTERRUPT, WDT_PRES_2PWR22, (void *)&wdt_func);
    
    while (1) {
    } /* while (1) */
} /* main() */

/***************************** End of File ************************************/
